@@ -33,21 +33,15 @@
 #include "ffcompat.h"
 #include <libswscale/swscale.h>
 
-
-/* xj5 seek modes */
-enum {  SEEK_ANY, ///< directly seek to givenvideo frame
-        SEEK_KEY, ///< seek to next keyframe after given frame.
-        SEEK_CONTINUOUS, ///< seek to keframe before this frame and advance to current frame.
-        SEEK_LIVESTREAM, ///< decode until next keyframe in a live-stream and set initial PTS offset; later decode cont. until PTS match
-};
-
+#ifndef MAX
+#define MAX(A,B) ( ( (A) > (B) ) ? (A) : (B) )
+#endif
 
 /* ffmpeg source */
 typedef struct {
   /* file specific decoder settings */
   int   want_ignstart; //< set before calling ff_open_movie()
   int   want_genpts;
-  int   seekflags;
   /* Video File Info */
   int   movie_width;  ///< original file geometry
   int   movie_height; ///< original file geometry
@@ -56,11 +50,12 @@ typedef struct {
 
   double duration;
   double framerate;
+  TimecodeRate tc;
   double file_frame_offset;
   long   frames;
   char  *current_file;
   /* helper variables */
-  double tpf;
+  int64_t tpf;
   int64_t avprev;
   int64_t stream_pts_offset;
   /* */
@@ -249,15 +244,16 @@ static void ff_init_moviebuffer(void *ptr) {
 
   if (ff->internal_buffer) free(ff->internal_buffer);
   ff_getbuffersize(ff, &numBytes);
+  assert(numBytes > 0);
   ff->internal_buffer = (uint8_t *) calloc(numBytes, sizeof(uint8_t));
   ff->buffer = ff->internal_buffer;
   ff->buf_width = ff->out_width;
   ff->buf_height = ff->out_height;
   if (!ff->buffer) {
 #ifdef WIN32
-    fprintf(stderr, "out of memory (trying to allocate %lu bytes)\n", numBytes); // XXX
+    fprintf(stderr, "out of memory (trying to allocate %lu bytes)\n", (long unsigned) numBytes);
 #else
-    fprintf(stderr, "out of memory (trying to allocate %zu bytes)\n", numBytes); // XXX
+    fprintf(stderr, "out of memory (trying to allocate %zu bytes)\n", numBytes);
 #endif
     exit(1);
   }
@@ -303,41 +299,60 @@ int ff_close_movie(void *ptr) {
   return (0);
 }
 
-static void ff_get_framerate(void *ptr, TimecodeRate *fr) {
-  ffst *ff = (ffst*)ptr;
-  AVStream *av_stream;
-  if (!ff->current_file || !ff->pFormatCtx) {
-    return;
-  }
-  av_stream = ff->pFormatCtx->streams[ff->videoStream];
-
-  if(av_stream->r_frame_rate.den && av_stream->r_frame_rate.num) {
-    fr->num = av_stream->r_frame_rate.num;
-    fr->den = av_stream->r_frame_rate.den;
- // if ((ff->framerate < 4 || ff->framerate > 100) && (av_stream->time_base.num && av_stream->time_base.den)) {
- //   fr->num = av_stream->time_base.den
- //   fr->den = av_stream->time_base.num;
- // }
-  } else {
-    fr->num = av_stream->time_base.den;
-    fr->den = av_stream->time_base.num;
-  }
-
-  fr->drop = 0;
-  if (floor(ff->framerate * 100.0) == 2997)
-    fr->drop = 1;
-}
-
-
 static void ff_set_framerate(ffst *ff) {
   AVStream *av_stream;
   av_stream = ff->pFormatCtx->streams[ff->videoStream];
 
-  if(av_stream->r_frame_rate.den && av_stream->r_frame_rate.num) {
-    ff->framerate = av_q2d(av_stream->r_frame_rate);
-    if ((ff->framerate < 4 || ff->framerate > 100) && (av_stream->time_base.num && av_stream->time_base.den))
-      ff->framerate = 1.0/av_q2d(av_stream->time_base);
+  ff->framerate = 0;
+  ff->tc.num = 0;
+  ff->tc.den = 1;
+
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(55, 0, 100) // 9cf788eca8ba (merge a75f01d7e0)
+  {
+    AVRational fr = av_stream->r_frame_rate;
+    if (fr.den > 0 && fr.num > 0) {
+      ff->framerate = av_q2d (av_stream->r_frame_rate);
+      ff->tc.num = fr.num;
+      ff->tc.den = fr.den;
+    }
   }
+#else
+  {
+    AVRational fr = av_stream_get_r_frame_rate (av_stream);
+    if (fr.den > 0 && fr.num > 0) {
+      ff->framerate = av_q2d (fr);
+      ff->tc.num = fr.num;
+      ff->tc.den = fr.den;
+    }
+  }
+#endif
+  if (ff->framerate < 1 || ff->framerate > 1000) {
+    AVRational fr = av_stream->avg_frame_rate;
+    if (fr.den > 0 && fr.num > 0) {
+      ff->framerate = av_q2d (fr);
+      ff->tc.num = fr.num;
+      ff->tc.den = fr.den;
+    }
+  }
+  if (ff->framerate < 1 || ff->framerate > 1000) {
+    AVRational fr = av_stream->time_base;
+    if (fr.den > 0 && fr.num > 0) {
+      ff->framerate = 1.0 / av_q2d (fr);
+      ff->tc.num = fr.den;
+      ff->tc.den = fr.num;
+    }
+  }
+  if (ff->framerate < 1 || ff->framerate > 1000) {
+    if (!want_quiet)
+      fprintf(stderr, "WARNING: cannot determine video-frame rate, using 25fps.\n");
+    ff->framerate = 25;
+    ff->tc.num = 25;
+    ff->tc.den = 1;
+  }
+
+  ff->tc.drop = 0;
+  if (floor(ff->framerate * 100.0) == 2997)
+    ff->tc.drop = 1;
 }
 
 int ff_open_movie(void *ptr, char *file_name, int render_fmt) {
@@ -363,8 +378,8 @@ int ff_open_movie(void *ptr, char *file_name, int render_fmt) {
   ff->framerate = ff->duration = ff->frames = 1;
   ff->file_frame_offset = 0.0;
   ff->videoStream = -1;
-  ff->tpf = 1.0;
-  ff->avprev = 0;
+  ff->tpf = 1;
+  ff->avprev = -1;
   ff->stream_pts_offset = AV_NOPTS_VALUE;
   ff->render_fmt = render_fmt;
 
@@ -375,15 +390,6 @@ int ff_open_movie(void *ptr, char *file_name, int render_fmt) {
       fprintf(stderr, "Cannot open video file %s\n", file_name);
     return (-1);
   }
-#if 1 /// XXX http is not neccesarily a live-stream!
-  // TODO: ff_seeflags(sf) API to set/get value
-  if (!strncmp(file_name, "http://", 7)) {
-    ff->seekflags = SEEK_LIVESTREAM;
-  } else {
-    ff->seekflags = SEEK_CONTINUOUS;
-  }
-  // TODO: live-stream: remember first pts as offset! -> don't use multiple-decoders for the same stream ?!
-#endif
 
   pthread_mutex_lock(&avcodec_lock);
   /* Retrieve stream information */
@@ -426,20 +432,21 @@ int ff_open_movie(void *ptr, char *file_name, int render_fmt) {
 
 #if 0 // DEBUG duration
   printf("DURATION frames from AVstream: %"PRIi64"\n", avs->nb_frames);
-  printf("DURATION duration from AVstream: %"PRIi64"\n", avs->duration /* * av_q2d(avs->r_frame_rate) * av_q2d(avs->time_base)*/);
-  printf("DURATION duration from FormatContext: %lu\n", ff->frames = ff->pFormatCtx->duration * ff->framerate / AV_TIME_BASE);
+  printf("DURATION duration from FormatContext: %.2f\n", ff->pFormatCtx->duration * ff->framerate / AV_TIME_BASE);
 #endif
 
-  if (avs->nb_frames != 0) {
+  if (avs->nb_frames > 0) {
     ff->frames = avs->nb_frames;
-  } else if (avs->duration != avs->duration && avs->duration != 0) // ???
-    ff->frames = avs->duration * av_q2d(avs->r_frame_rate) * av_q2d(avs->time_base);
-  else {
+    ff->duration = ff->frames / ff->framerate;
+  } else {
+    ff->duration = ff->pFormatCtx->duration / (double)AV_TIME_BASE;
     ff->frames = ff->pFormatCtx->duration * ff->framerate / AV_TIME_BASE;
   }
-  ff->duration = (double) avs->duration * av_q2d(avs->time_base);
+
+  const AVRational fr_Q = { ff->tc.den, ff->tc.num };
+  ff->tpf = av_rescale_q (1, fr_Q, avs->time_base);
   }
-  ff->tpf = 1.0/(av_q2d(ff->pFormatCtx->streams[ff->videoStream]->time_base)*ff->framerate);
+
   ff->file_frame_offset = ff->framerate*((double) ff->pFormatCtx->start_time/ (double) AV_TIME_BASE);
 
   if (want_verbose) {
@@ -497,7 +504,7 @@ int ff_open_movie(void *ptr, char *file_name, int render_fmt) {
   }
   pthread_mutex_unlock(&avcodec_lock);
 
-  if (!(ff->pFrame = avcodec_alloc_frame())) {
+  if (!(ff->pFrame = av_frame_alloc())) {
     if (!want_quiet)
       fprintf(stderr, "Cannot allocate video frame buffer\n");
     avcodec_close(ff->pCodecCtx);
@@ -505,7 +512,7 @@ int ff_open_movie(void *ptr, char *file_name, int render_fmt) {
     return(-1);
   }
 
-  if (!(ff->pFrameFMT = avcodec_alloc_frame())) {
+  if (!(ff->pFrameFMT = av_frame_alloc())) {
     if (!want_quiet)
       fprintf(stderr, "Cannot allocate display frame buffer\n");
     av_free(ff->pFrame);
@@ -520,157 +527,161 @@ int ff_open_movie(void *ptr, char *file_name, int render_fmt) {
   return(0);
 }
 
-static void reset_video_head(ffst *ff, AVPacket *packet) {
-  int frameFinished = 0;
-  if (!want_quiet)
-    fprintf(stderr, "Resetting decoder - seek/playhead rewind.\n");
-#if LIBAVFORMAT_BUILD < 4617
-  av_seek_frame(ff->pFormatCtx, ff->videoStream, 0);
-#else
-  av_seek_frame(ff->pFormatCtx, ff->videoStream, 0, AVSEEK_FLAG_BACKWARD);
-#endif
-  avcodec_flush_buffers(ff->pCodecCtx);
+static uint64_t parse_pts_from_frame (AVFrame *f) {
+  uint64_t pts = AV_NOPTS_VALUE;
+  static uint8_t pts_warn = 0; // should be per decoder
 
-  while (!frameFinished) {
-    av_read_frame(ff->pFormatCtx, packet);
-    if(packet->stream_index == ff->videoStream)
-#if LIBAVCODEC_VERSION_MAJOR < 52 || (LIBAVCODEC_VERSION_MAJOR == 52 && LIBAVCODEC_VERSION_MINOR < 21)
-    avcodec_decode_video(ff->pCodecCtx, ff->pFrame, &frameFinished, packet->data, packet->size);
-#else
-    avcodec_decode_video2(ff->pCodecCtx, ff->pFrame, &frameFinished, packet);
-#endif
-    if(packet->data) av_free_packet(packet);
+  pts = AV_NOPTS_VALUE;
+
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(51, 49, 100)
+  if (pts == AV_NOPTS_VALUE) {
+    pts = av_frame_get_best_effort_timestamp (f);
+    if (pts != AV_NOPTS_VALUE) {
+      if (!(pts_warn & 1) && want_verbose)
+	fprintf(stderr, "PTS: Best effort.\n");
+      pts_warn |= 1;
+    }
   }
-  ff->avprev = 0;
+#else
+#warning building with libavutil < 51.49.100 is highly discouraged
+#endif
+
+  if (pts == AV_NOPTS_VALUE) {
+    pts = f->pkt_pts;
+    if (pts != AV_NOPTS_VALUE) {
+      if (!(pts_warn & 2) && want_verbose)
+	fprintf(stderr, "Used PTS from packet instead frame's PTS.\n");
+      pts_warn |= 2;
+    }
+  }
+
+  if (pts == AV_NOPTS_VALUE) {
+    pts = f->pts; // sadly bogus with many codecs :(
+    if (pts != AV_NOPTS_VALUE) {
+      if (!(pts_warn & 8) && want_verbose)
+	fprintf(stderr, "Used AVFrame assigned pts (instead frame PTS).\n");
+      pts_warn |= 8;
+    }
+  }
+
+  if (pts == AV_NOPTS_VALUE) {
+    pts = f->pkt_dts;
+    if (pts != AV_NOPTS_VALUE) {
+      if (!(pts_warn & 4) && want_verbose)
+	fprintf(stderr, "Used decode-timestamp from packet (instead frame PTS).\n");
+      pts_warn |= 4;
+    }
+  }
+
+  return pts;
 }
 
-// TODO: set this high (>1000) if transport stopped and to a low value (<100) if transport is running.
-#define MAX_CONT_FRAMES (1000)
-
-static int my_seek_frame (ffst *ff, AVPacket *packet, int64_t timestamp) {
+static int my_seek_frame (ffst *ff, AVPacket *packet, int64_t framenumber) {
   AVStream *v_stream;
-  int rv = 1;
-  int64_t mtsb = 0;
-  int frameFinished;
-  int nolivelock = 0;
-  static int ffdebug = 0;
+  int rv = 0;
+  int64_t timestamp;
 
   if (ff->videoStream < 0) return (0);
   v_stream = ff->pFormatCtx->streams[ff->videoStream];
 
-  if (ff->want_ignstart)  // timestamps in the file start counting at ..->start_time
-    timestamp += (int64_t) rint(ff->framerate*((double)ff->pFormatCtx->start_time / (double)AV_TIME_BASE));
+  if (ff->want_ignstart)
+    framenumber += (int64_t) rint(ff->framerate * ((double)ff->pFormatCtx->start_time / (double)AV_TIME_BASE));
 
-  // TODO: assert  0 <= timestamp + ts_offset - (..->start_time)  < length
+  if (framenumber < 0 || framenumber >= ff->frames) {
+    return -1;
+  }
 
-#if LIBAVFORMAT_BUILD > 4629 // verify this version
-  timestamp = av_rescale_q(timestamp, c1_Q, v_stream->time_base);
-  timestamp = av_rescale_q(timestamp, c1_Q, v_stream->r_frame_rate); //< timestamp/=framerate;
-#endif
+  const AVRational fr_Q = { ff->tc.den, ff->tc.num };
+  timestamp = av_rescale_q(framenumber, fr_Q, v_stream->time_base);
 
-#if LIBAVFORMAT_BUILD < 4617
-  rv= av_seek_frame(ff->pFormatCtx, ff->videoStream, timestamp / framerate * 1000000LL);
-#else
-  if (ff->seekflags == SEEK_ANY) {
-    rv = av_seek_frame(ff->pFormatCtx, ff->videoStream, timestamp, AVSEEK_FLAG_ANY | AVSEEK_FLAG_BACKWARD) ;
-    avcodec_flush_buffers(ff->pCodecCtx);
-  } else if (ff->seekflags == SEEK_KEY) {
+  if (ff->avprev == timestamp) {
+    return 0;
+  }
+
+  if (ff->avprev < 0 || ff->avprev >= timestamp || ((ff->avprev + 32 * ff->tpf) < timestamp)) {
     rv = av_seek_frame(ff->pFormatCtx, ff->videoStream, timestamp, AVSEEK_FLAG_BACKWARD) ;
-    avcodec_flush_buffers(ff->pCodecCtx);
-  } else if (ff->seekflags == SEEK_LIVESTREAM) {
-  } else /* SEEK_CONTINUOUS */ if (ff->avprev >= timestamp || ((ff->avprev + 32*ff->tpf) < timestamp)) {
-    // NOTE: only seek if last-frame is less then 32 frames behind
-    // else read continuously until we get there :D
-    // FIXME 32: use keyframes interval of video file or cmd-line-arg as threshold.
-    // TODO: do we know if there is a keyframe inbetween now (my_avprev)
-    // and the frame to seek to?? - if so rather seek to that frame than read until then.
-    // and if no keyframe in between my_avprev and ts - prevent backwards seeks even if
-    // timestamp-my_avprev > threshold! - Oh well.
-
-    // seek to keyframe *BEFORE* this frame
-    rv= av_seek_frame(ff->pFormatCtx, ff->videoStream, timestamp, AVSEEK_FLAG_BACKWARD) ;
-    avcodec_flush_buffers(ff->pCodecCtx);
-  }
-#endif
-
-  ff->avprev = timestamp;
-  if (rv < 0) return (0); // seek failed.
-
-read_frame:
-  nolivelock++;
-  if(av_read_frame(ff->pFormatCtx, packet) < 0) {
-    if (!want_quiet) {
-      fprintf(stderr, "Reached movie end\n");
+    if (ff->pCodecCtx->codec->flush) {
+      avcodec_flush_buffers(ff->pCodecCtx);
     }
-    return (0);
   }
-#if LIBAVFORMAT_BUILD >= 4616
-  if (av_dup_packet(packet) < 0) {
-    if (!want_quiet) {
-      fprintf(stderr, "can not allocate packet\n");
+
+  ff->avprev = -1;
+
+  if (rv < 0) {
+    return -1;
+  }
+
+  int bailout = 600;
+  int decoded = 0;
+  while (bailout > 0) {
+    int err;
+    if ((err = av_read_frame (ff->pFormatCtx, packet)) < 0) {
+      if (err != AVERROR_EOF) {
+	av_free_packet (packet);
+	return -1;
+      } else {
+	--bailout;
+      }
+    }
+    if(packet->stream_index != ff->videoStream) {
+      av_free_packet (packet);
+      continue;
     }
-    goto read_frame;
-  }
-#endif
-  if(packet->stream_index != ff->videoStream) {
-    if (packet->data) av_free_packet(packet);
-    goto read_frame;
-  }
-  /* backwards compatible - no cont. seeking (seekmode ANY or KEY ; cmd-arg: -K, -k)
-   * do we want a AVSEEK_FLAG_ANY + SEEK_CONTINUOUS option ?? not now.  */
-#if LIBAVFORMAT_BUILD < 4617
-  return (1);
-#endif
 
-  if (   ff->seekflags != SEEK_CONTINUOUS
-      && ff->seekflags != SEEK_LIVESTREAM
-     ) return (1);
-
-  mtsb = packet->pts;
-  if (mtsb == AV_NOPTS_VALUE) {
-    mtsb = packet->dts;
-    if (ffdebug == 0) { ffdebug = 1; if (!want_quiet) fprintf(stderr, "WARNING: video file does not report pts information.\n         resorting to ffmpeg decompression timestamps.\n         consider to transcode the file or use the --genpts option.\n"); }
-  }
-  if (mtsb == AV_NOPTS_VALUE) {
-    if (ffdebug < 2) { ffdebug = 2; if (!want_quiet) fprintf(stderr, "ERROR: neither the video file nor the ffmpeg decoder were able to\n       provide a video frame timestamp."); }
-    av_free_packet(packet);
-    return (0);
-  }
-
-#if 1 // experimental
-  /* remember live-stream PTS offset */
-  if (   ff->seekflags == SEEK_LIVESTREAM
-      && mtsb != AV_NOPTS_VALUE
-      && ff->stream_pts_offset == AV_NOPTS_VALUE
-      && packet->flags&AV_PKT_FLAG_KEY)
-  {
-    ff->stream_pts_offset = mtsb;
-  }
-
-  if (ff->seekflags == SEEK_LIVESTREAM) {
-    if (ff->stream_pts_offset != AV_NOPTS_VALUE)
-      mtsb -= ff->stream_pts_offset;
-    else
-      mtsb = AV_NOPTS_VALUE;
-  }
-#endif
-
-  if (mtsb >= timestamp) {
-    return (1); // ok!
-  }
-
-  /* skip to next frame */
-
-#if LIBAVCODEC_VERSION_MAJOR < 52 || ( LIBAVCODEC_VERSION_MAJOR == 52 && LIBAVCODEC_VERSION_MINOR < 21)
-  avcodec_decode_video(ff->pCodecCtx, ff->pFrame, &frameFinished, packet->data, packet->size);
+    int frameFinished = 0;
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(52, 21, 0)
+    err = avcodec_decode_video (ff->pCodecCtx, ff->pFrame, &frameFinished, packet->data, packet->size);
 #else
-  avcodec_decode_video2(ff->pCodecCtx, ff->pFrame, &frameFinished, packet);
+    err = avcodec_decode_video2 (ff->pCodecCtx, ff->pFrame, &frameFinished, packet);
 #endif
-  av_free_packet(packet);
-  if (!frameFinished) goto read_frame;
-  if (nolivelock < MAX_CONT_FRAMES) goto read_frame;
-  reset_video_head(ff, packet);
-  return (0); // seek failed.
+    av_free_packet (packet);
+
+    if (err < 0) {
+      return -10;
+    }
+
+    if (!frameFinished) {
+      --bailout;
+      continue;
+    }
+
+    int64_t pts = parse_pts_from_frame (ff->pFrame);
+
+    if (pts == AV_NOPTS_VALUE) {
+      return -7;
+    }
+
+    const int64_t prefuzz = ff->tpf > 10 ? 1 : 0;
+    if (pts + prefuzz >= timestamp) {
+      if (pts - timestamp < ff->tpf) {
+	ff->avprev = pts;
+	return 0; // OK
+      }
+      // Cannot reliably seek to target frame
+      if (decoded == 0) {
+	if (want_verbose)
+	  fprintf(stdout, " PTS mismatch want: %"PRId64" got: %"PRId64" -> re-seek\n", timestamp, pts);
+	// re-seek - make a guess, since we don't know the keyframe interval
+	rv = av_seek_frame(ff->pFormatCtx, ff->videoStream, MAX(0, timestamp - ff->tpf * 25), AVSEEK_FLAG_BACKWARD) ;
+	if (ff->pCodecCtx->codec->flush) {
+	  avcodec_flush_buffers(ff->pCodecCtx);
+	}
+	if (rv < 0) {
+	  return -3;
+	}
+	--bailout;
+	++decoded;
+	continue;
+      }
+      if (!want_quiet)
+	fprintf(stderr, " PTS mismatch want: %"PRId64" got: %"PRId64" -> fail\n", timestamp, pts); // XXX
+      return -2;
+    }
+
+    --bailout;
+    ++decoded;
+  }
+  return -5;
 }
 
 /**
@@ -688,53 +699,23 @@ read_frame:
 int ff_render(void *ptr, unsigned long frame,
     uint8_t* buf, int w, int h, int xoff, int xw, int ys) {
   ffst *ff = (ffst*) ptr;
-  int frameFinished = 0;
-  int64_t timestamp = (int64_t) frame;
 
   if (ff->buffer == ff->internal_buffer && (ff->buf_width <= 0 || ff->buf_height <= 0)) {
     ff_init_moviebuffer(ff);
   }
 
-  if (ff->pFrameFMT && ff->pFormatCtx && my_seek_frame(ff, &ff->packet, timestamp)) {
-    while (1) { /* Decode video frame */
-      frameFinished = 0;
-      if(ff->packet.stream_index == ff->videoStream)
-#if LIBAVCODEC_VERSION_MAJOR < 52 || ( LIBAVCODEC_VERSION_MAJOR == 52 && LIBAVCODEC_VERSION_MINOR < 21)
-	avcodec_decode_video(ff->pCodecCtx, ff->pFrame, &frameFinished, ff->packet.data, ff->packet.size);
-#else
-	avcodec_decode_video2(ff->pCodecCtx, ff->pFrame, &frameFinished, &ff->packet);
-#endif
-      if(frameFinished) { /* Convert the image from its native format to FMT */
-	ff->pSWSCtx = sws_getCachedContext(ff->pSWSCtx, ff->pCodecCtx->width, ff->pCodecCtx->height, ff->pCodecCtx->pix_fmt, ff->out_width, ff->out_height, ff->render_fmt, SWS_BICUBIC, NULL, NULL, NULL);
-	sws_scale(ff->pSWSCtx, (const uint8_t * const*) ff->pFrame->data, ff->pFrame->linesize, 0, ff->pCodecCtx->height, ff->pFrameFMT->data, ff->pFrameFMT->linesize);
-	av_free_packet(&ff->packet);
-	break;
-      } else  {
-	if(ff->packet.data) av_free_packet(&ff->packet);
-	if(av_read_frame(ff->pFormatCtx, &ff->packet) < 0) {
-	  if (!want_quiet) fprintf(stderr, "read error!\n");
-	  reset_video_head(ff, &ff->packet);
-	  render_empty_frame(ff, buf, w, h, xoff, ys);
-	  break;
-	}
-#if LIBAVFORMAT_BUILD >= 4616
-	if (av_dup_packet(&ff->packet) < 0) {
-	  if (!want_quiet)
-	    fprintf(stderr, "Cannot allocate packet\n");
-	  break;
-	}
-#endif
-      }
-    } /* end while !frame_finished */
-  } else {
-    if (ff->pFrameFMT && !want_quiet) fprintf( stderr, "frame seek unsucessful (frame: %lu).\n", frame);
+  if (ff->pFrameFMT && ff->pFormatCtx && !my_seek_frame(ff, &ff->packet, frame)) {
+    ff->pSWSCtx = sws_getCachedContext(ff->pSWSCtx, ff->pCodecCtx->width, ff->pCodecCtx->height, ff->pCodecCtx->pix_fmt, ff->out_width, ff->out_height, ff->render_fmt, SWS_BICUBIC, NULL, NULL, NULL);
+    sws_scale(ff->pSWSCtx, (const uint8_t * const*) ff->pFrame->data, ff->pFrame->linesize, 0, ff->pCodecCtx->height, ff->pFrameFMT->data, ff->pFrameFMT->linesize);
+    return 0;
   }
 
-  if (!frameFinished) {
-    render_empty_frame(ff, buf, w, h, xoff, ys);
-    return -1;
+  if (ff->pFrameFMT && !want_quiet) {
+    fprintf( stderr, "frame seek unsucessful (frame: %lu).\n", frame);
   }
-  return 0;
+
+  render_empty_frame(ff, buf, w, h, xoff, ys);
+  return -1;
 }
 
 void ff_get_info(void *ptr, VInfo *i) {
@@ -751,10 +732,9 @@ void ff_get_info(void *ptr, VInfo *i) {
     ff_getbuffersize(ptr, &i->buffersize);
   else
     i->buffersize = 0;
-  i->frames = ff->frames; // ff->duration * ff->framerate;
+  i->frames = ff->frames;
 
-  //fl2ratio(&(i->framerate->num), &(i->framerate->den), ff->framerate);
-  ff_get_framerate(ptr, &i->framerate);
+  memcpy(&i->framerate, &ff->tc, sizeof(TimecodeRate));
 }
 
 void ff_get_info_canonical(void *ptr, VInfo *i, int w, int h) {
@@ -770,7 +750,6 @@ void ff_get_info_canonical(void *ptr, VInfo *i, int w, int h) {
 void ff_create(void **ff) {
   (*((ffst**)ff)) = (ffst*) calloc(1, sizeof(ffst));
   (*((ffst**)ff))->render_fmt = PIX_FMT_RGB24;
-  (*((ffst**)ff))->seekflags = SEEK_CONTINUOUS;
   (*((ffst**)ff))->want_ignstart = 0;
   (*((ffst**)ff))->want_genpts = 0;
   (*((ffst**)ff))->packet.data = NULL;
