@@ -1,7 +1,7 @@
 /*
    harvid -- http ardour video daemon
 
-   Copyright (C) 2008-2013 Robin Gareus <robin@gareus.org>
+   Copyright (C) 2008-2014 Robin Gareus <robin@gareus.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -306,7 +306,11 @@ int main (int argc, char **argv) {
   program_name = argv[0];
   struct stat sb;
   int cfg_uid, cfg_gid;
-  char *docroot = "/" ;
+#ifdef HAVE_WINDOWS
+  char *docroot = "C:\\";
+#else
+  char *docroot = "/";
+#endif
   debug_level = DLOG_WARNING;
   int exitstatus = 0;
 
@@ -404,6 +408,7 @@ char *hdl_homepage_html (CONN *c) {
   off+=snprintf(msg+off, HPSIZE-off, "<title>harvid</title></head>\n");
   off+=snprintf(msg+off, HPSIZE-off, HTMLBODY);
   off+=snprintf(msg+off, HPSIZE-off, CENTERDIV);
+  off+=snprintf(msg+off, HPSIZE-off, "<div style=\"text-align:center;\"><h1>Harvid - HTTP Ardour Video Daemon</h1></div>\n");
   off+=snprintf(msg+off, HPSIZE-off, "<div style=\"float:left;margin:0 2em;\"><h2>Built-in handlers</h2>\n");
   off+=snprintf(msg+off, HPSIZE-off, "<ul>");
   if (cfg_usermask & USR_INDEX) {
