@@ -1,7 +1,7 @@
 /*
    This file is part of harvid
 
-   Copyright (C) 2008-2013 Robin Gareus <robin@gareus.org>
+   Copyright (C) 2008-2014 Robin Gareus <robin@gareus.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -160,6 +160,10 @@ static int parse_http_query(CONN *c, char *query, httpheader *h, ics_request_arg
     if (qps.fn) {
       a->file_name = malloc(1+strlen(c->d->docroot)+strlen(qps.fn)*sizeof(char));
       sprintf(a->file_name, "%s%s", c->d->docroot, qps.fn);
+#ifdef HAVE_WINDOWS
+      char *tmp;
+      while (tmp = strchr(a->file_name, '/')) *tmp = '\\';
+#endif
       a->file_qurl = qps.fn;
     }
 
@@ -337,6 +341,10 @@ void ics_http_handler(
     char *dp = url_unescape(&(path[7]), 0, NULL);
     char *abspath = malloc((strlen(c->d->docroot) + strlen(dp) + 2) * sizeof(char));
     sprintf(abspath, "%s/%s", c->d->docroot, dp);
+#ifdef HAVE_WINDOWS
+      char *tmp;
+      while (tmp = strchr(abspath, '/')) *tmp = '\\';
+#endif
     if (! (cfg_usermask & USR_INDEX)) {
       httperror(c->fd, 403, NULL, NULL);
     } else if (!dp || check_path(dp)) {
@@ -348,6 +356,7 @@ void ics_http_handler(
       dlog(DLOG_WARNING, "CON: permission denied for dir: '%s'\n", abspath);
       httperror(c->fd, 403, NULL, NULL);
     } else {
+      debugmsg(DEBUG_ICS, "indexing dir: '%s'\n", abspath);
       ics_request_args a;
       char base_url[1024];
       struct queryparserstate qps = {&a, NULL, 0};
